/*
 * Copyright (c) 2007 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: HierarchicalWorkflowTestCase.java,v 1.4 2007/04/24 01:14:02 sshirasu Exp $
 */
package edu.indiana.extreme.xbaya.test;

import java.awt.Point;
import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.security.cert.X509Certificate;
import java.util.List;

import javax.xml.namespace.QName;

import org.gpel.client.GcInstance;
import org.gpel.client.security.GpelUserX509Credential;
import org.ietf.jgss.GSSCredential;

import xsul.lead.LeadContextHeader;
import xsul.wsif.WSIFMessage;
import xsul5.MLogger;
import xsul5.wsdl.WsdlDefinitions;
import edu.indiana.extreme.xbaya.XBayaException;
import edu.indiana.extreme.xbaya.component.Component;
import edu.indiana.extreme.xbaya.component.ComponentException;
import edu.indiana.extreme.xbaya.component.registry.ComponentRegistryException;
import edu.indiana.extreme.xbaya.component.registry.LocalComponentRegistry;
import edu.indiana.extreme.xbaya.component.system.InputComponent;
import edu.indiana.extreme.xbaya.component.system.OutputComponent;
import edu.indiana.extreme.xbaya.component.ws.WSComponentPort;
import edu.indiana.extreme.xbaya.component.ws.WorkflowComponent;
import edu.indiana.extreme.xbaya.file.XBayaPathConstants;
import edu.indiana.extreme.xbaya.gpel.GPELClient;
import edu.indiana.extreme.xbaya.gpel.GPELClient.WorkflowType;
import edu.indiana.extreme.xbaya.gpel.component.GPELRegistry;
import edu.indiana.extreme.xbaya.graph.Graph;
import edu.indiana.extreme.xbaya.graph.GraphException;
import edu.indiana.extreme.xbaya.graph.Node;
import edu.indiana.extreme.xbaya.graph.system.InputNode;
import edu.indiana.extreme.xbaya.graph.system.OutputNode;
import edu.indiana.extreme.xbaya.lead.LEADWorkflowInvoker;
import edu.indiana.extreme.xbaya.lead.LeadContextHeaderHelper;
import edu.indiana.extreme.xbaya.myproxy.MyProxyClient;
import edu.indiana.extreme.xbaya.security.XBayaSecurity;
import edu.indiana.extreme.xbaya.test.service.adder.Adder;
import edu.indiana.extreme.xbaya.test.util.WorkflowCreator;
import edu.indiana.extreme.xbaya.util.XMLUtil;
import edu.indiana.extreme.xbaya.wf.Workflow;

/**
 * @author Satoshi Shirasuna
 */
public class HierarchicalWorkflowTestCase extends XBayaTestCase {

    private static final MLogger logger = MLogger.getLogger();

    private GPELClient gpelClient;

    private WorkflowCreator workflowCreator;

    private GPELRegistry gpelComponentRegistry;

    private LocalComponentRegistry componentRegistry;

    /**
     * @see edu.indiana.extreme.xbaya.test.XBayaTestCase#setUp()
     */
    @Override
    protected void setUp() throws Exception {
        super.setUp();

        this.workflowCreator = new WorkflowCreator();

        // GPEL Setup
        X509Certificate[] trustedCertificates = XBayaSecurity
                .getTrustedCertificates();
        MyProxyClient client = new MyProxyClient(this.configuration
                .getMyProxyServer(), this.configuration.getMyProxyPort(),
                this.configuration.getMyProxyUsername(), this.configuration
                        .getMyProxyPassphrase(), this.configuration
                        .getMyProxyLifetime());
        client.load();
        GSSCredential proxy = client.getProxy();
        GpelUserX509Credential credential = new GpelUserX509Credential(proxy,
                trustedCertificates);
        this.gpelClient = new GPELClient(this.configuration.getGPELEngineURL(),
                credential);

        this.gpelComponentRegistry = new GPELRegistry(this.gpelClient,
                WorkflowType.TEMPLATE, 100);
        this.componentRegistry = new LocalComponentRegistry(
                XBayaPathConstants.WSDL_DIRECTORY);
    }

    /**
     * @throws IOException
     * @throws XBayaException
     */
    public void test() throws IOException, XBayaException {
        Workflow subWorkflow = this.workflowCreator.createSimpleMathWorkflow();

        URI subWorkflowID = this.gpelClient.createScriptAndDeploy(subWorkflow,
                false);
        Workflow workflow = createHierarchicalWorkflow(subWorkflowID);
        File file = new File(this.temporalDirectory, "hierarchical-test.xwf");
        XMLUtil.saveXML(workflow.toXML(), file);

        URI workflowTemplateID = this.gpelClient.createScriptAndDeploy(
                workflow, false);
        logger.info("workflowTemplateID: " + workflowTemplateID);

        // Instantiate the workflow template.
        GcInstance instance = this.gpelClient.instantiate(workflow,
                this.configuration.getDSCURL());

        // ID to retrieve the workflow instance
        URI instanceID = instance.getInstanceId();
        logger.info("instanceID: " + instanceID);

        // Start the workflow instance.
        WsdlDefinitions wsdl = this.gpelClient.start(instance);

        // Create lead context.
        LeadContextHeaderHelper leadContextHelper = new LeadContextHeaderHelper();
        leadContextHelper.setXBayaConfiguration(this.configuration);
        LeadContextHeader leadContext = leadContextHelper
                .getLeadContextHeader();

        URI messageBoxURL = null;
        if (this.configuration.isPullMode()) {
            messageBoxURL = this.configuration.getMessageBoxURL();
        }

        // Create a invoke to invoke the workflow instance.
        LEADWorkflowInvoker invoker = new LEADWorkflowInvoker(wsdl,
                leadContext, messageBoxURL);

        // Set the input values to the invoker.
        // Get the input information
        List<WSComponentPort> inputs = workflow.getInputs();

        for (WSComponentPort input : inputs) {
            // Show the information of each input.

            // Name
            String name = input.getName();
            logger.info("name: " + name);

            // Type
            QName type = input.getType();
            logger.info("type: " + type);

            String defaultValue = input.getDefaultValue();
            logger.info("defaultValue: " + defaultValue);

            // Set a value to each input.
            input.setValue(defaultValue);
        }
        invoker.setInputs(inputs);

        // Invoke the workflow. This will block, so you may want to do it in a
        // thread.
        boolean success = invoker.invoke();
        logger.info("success: " + success);

        // We don't need to wait for the outputs.

        if (success) {
            List<WSComponentPort> outputs = invoker.getOutputs();
            for (WSComponentPort output : outputs) {
                String name = output.getName();
                logger.info("name: " + name);
                Object value = output.getValue();
                logger.info("value: " + value);
            }
        } else {
            WSIFMessage fault = invoker.getFault();
            logger.info("fault: " + fault);
        }

    }

    private Workflow createHierarchicalWorkflow(URI subWorkflowID)
            throws ComponentRegistryException, ComponentException,
            GraphException {
        Workflow workflow = new Workflow();

        // Name, description
        workflow.setName("Hierarchical workflow");
        workflow.setDescription("Hierarchical workflow");

        Graph graph = workflow.getGraph();

        // Input 1
        InputComponent inputComponent = new InputComponent();
        InputNode input1 = (InputNode) workflow.addNode(inputComponent);
        input1.setPosition(new Point(50, 50));

        // Input 2
        InputNode input2 = (InputNode) workflow.addNode(inputComponent);
        input2.setPosition(new Point(50, 150));

        InputNode input3 = (InputNode) workflow.addNode(inputComponent);
        input3.setPosition(new Point(50, 250));

        // Adder node
        Component adderComp = this.componentRegistry
                .getComponent(Adder.WSDL_PATH);
        Node adder = workflow.addNode(adderComp);
        adder.setPosition(new Point(200, 100));

        // Sub workflow
        WorkflowComponent subWorkflowComponent = this.gpelComponentRegistry
                .getComponent(subWorkflowID);
        Node subWorkflow = workflow.addNode(subWorkflowComponent);
        subWorkflow.setPosition(new Point(350, 150));

        // Output
        OutputComponent outputComponent = new OutputComponent();
        OutputNode outParamNode = (OutputNode) workflow
                .addNode(outputComponent);
        outParamNode.setPosition(new Point(800, 150));

        // Connect ports
        graph.addEdge(input1.getOutputPort(0), adder.getInputPort(0));
        graph.addEdge(input2.getOutputPort(0), adder.getInputPort(1));
        graph.addEdge(adder.getOutputPort(0), subWorkflow.getInputPort(0));
        graph.addEdge(input3.getOutputPort(0), subWorkflow.getInputPort(1));
        graph.addEdge(subWorkflow.getOutputPort(0), outParamNode
                .getInputPort(0));

        // Set the default values
        // This needs to be after connection.
        input1.setDefaultValue("2");
        input2.setDefaultValue("3");
        input3.setDefaultValue("4");
        return workflow;
    }

}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2007 The Trustees of Indiana University. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
